#!/usr/bin/env python

# This script produces downscaled images from micrographs in a Relion project (Relion version > 3.1) which can be used for micrograph selection.
# Run this python script as Relion External job. Input is a micrograph star file from the Relion project. By default the script performs 8x binning. 
# Optional arguments are the input micrograph pixel size, angpix, and the rescaled image pixel size, rescale_angpix.
# Downscaled images are by default stored in a folder Processed in the External job folder  
# The output micrographs_processed.star file contains the fields  of the input file and an additional field, _rlnImageName, for the location of the  downscaled image. 
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import subprocess, sys, argparse, os, re

"""VARIABLES >>>"""
print('running ...')
parser = argparse.ArgumentParser()
parser.add_argument("-i", "--inputmic", "--in_mics", type=str, help = "Input micrographs")
parser.add_argument("-o", "--output", type=str, help = "Output folder (full path)")
parser.add_argument("-ap", "--angpix", type=int, default=1, help = "pixel size in micrograph")
parser.add_argument("-rap", "--rescale_angpix", type=int, default=8, help = "pixel size in micrograph")
args, unknown = parser.parse_known_args()

inargsMics =str(args.inputmic)
outargsPath =str( args.output)
outargsProc = outargsPath+'Processed/'
pix_size=float(args.angpix)
repix_size=float(args.rescale_angpix)
outargsStar = outargsPath+'micrographs_processed.star'

print(inargsMics)

if not os.path.exists(outargsProc):
	os.makedirs(outargsProc)
"""<<< VARIABLES"""

"""Read Header >>>"""
header_dfields = []
foundheader = False
foundloop = False
ln = 0
with open(inargsMics, 'r') as f:
        for l in f:
            if l.lstrip().startswith("data_micrographs"):
              foundheader = True
              print("foundheader")
            if (l.lstrip().startswith("_") and foundheader == True):
                foundloop = True
                head = l.split('#')[0].strip()
                header_dfields.append(head)
                lastheaderl = ln + 1
                if l.lstrip().startswith("_rlnImageName"):
                  print("Field _rlnImageName found. No micrograph file or already processed. Terminating script")
                  sys.exit() 
            if (foundloop == True and not l.startswith("_")):
                break
            ln += 1

joined_string = ' '.join([str(fi) for fi in header_dfields])
print(joined_string)
"""<<<Read Header"""


"""write star header>>>"""
fout=open(outargsStar,"w+")
with open(inargsMics, 'r') as f:
  for i in range(lastheaderl):
    line = next(f)
    fout.write(line)
  fout.write("_rlnImageName \n")
  f.close()
"""<<<write star header"""

"""processing>>>"""
out = subprocess.check_output(str('relion_star_printtable ')+inargsMics+str(' data_micrographs _rlnMicrographName'), shell=True)
outfull = subprocess.check_output(str('relion_star_printtable ')+inargsMics+" data_micrographs "+joined_string, shell=True)
micdat=outfull.splitlines()
mic=out.splitlines()

count=0 
for image in mic:
  micnamep=(mic[count].decode("utf-8"))
  micnameo=os.path.basename(micnamep)
  print(micnameo)
  outmrc=micnameo[:-4]+"_coarse.mrc"
  print(outmrc)
  cmd=str(' relion_image_handler --i ')+micnamep+str(' --angpix ')+str(pix_size)+str(' --rescale_angpix ')+str(repix_size)+str(' --o ')+outargsProc+outmrc
  os.system(cmd) 
  fout.write((micdat[count].decode("utf-8"))+" "+outargsProc+outmrc+" \n")
  count+=1
fout.close()
"""<<<processing"""

"""Finishing up >>>"""
f=open(outargsPath+"RELION_OUTPUT_NODES.star","w+")
f.write("data_output_nodes\nloop_\n_rlnPipeLineNodeName #1\n _rlnPipeLineNodeType #2\n"+outargsStar+" 1\n")
f.close()
f=open(outargsPath+"RELION_JOB_EXIT_SUCCESS","w+")
f.close()

print('All done!')
"""<<< Finishing up"""
